@@ -36,20 +36,23 @@ def merge_config_logs(experiment_dir: str, all_run_ids: list) -> None:
     # Need to make sure that run_ids & experiment folder match!
     for run_id in all_run_ids:
         for f in all_folders:
-            if f[len(experiment_dir) + 9 :] == run_id:
+            path, file = os.path.split(f)
+            if file == run_id:
                 hyperp_results_folder.append(f)
                 continue
-
     # Collect all paths to the .hdf5 file
     log_paths = []
+
     for i in range(len(hyperp_results_folder)):
         log_d_t = os.path.join(hyperp_results_folder[i], "logs/")
         for file in os.listdir(log_d_t):
             fname, fext = os.path.splitext(file)
-            if file.endswith(".hdf5") and fname in all_run_ids:
-                log_paths.append(os.path.join(log_d_t, file))
+            if file.endswith(".hdf5"):
+                if fname in all_run_ids or fname == "log":
+                    log_paths.append(os.path.join(log_d_t, file))
 
     # Merge individual run results into a single hdf5 file
     assert len(log_paths) == len(all_run_ids)
+
     meta_log_fname = os.path.join(experiment_dir, "meta_log.hdf5")
     merge_hdf5_files(meta_log_fname, log_paths, file_ids=all_run_ids)
