@@ -1,8 +1,16 @@
 from .stats_log import StatsLog
 from .tboard_log import TboardLog
+from .wandb_log import WandbLog
 from .model_log import ModelLog
 from .extra_log import ExtraLog
 from .figure_log import FigureLog
 
 
-__all__ = ["StatsLog", "TboardLog", "ModelLog", "ExtraLog", "FigureLog"]
+__all__ = [
+    "StatsLog",
+    "TboardLog",
+    "WandbLog",
+    "ModelLog",
+    "ExtraLog",
+    "FigureLog",
+]
