@@ -36,15 +36,29 @@ class ModelLog(object):
         self.ckpt_time_to_track = ckpt_time_to_track
         self.top_k_metric_name = top_k_metric_name
         self.top_k_minimize_metric = top_k_minimize_metric
+        self.seed_id = seed_id
+
+        if self.save_every_k_ckpt:
+            assert self.ckpt_time_to_track is not None
+        if self.save_top_k_ckpt:
+            assert self.ckpt_time_to_track is not None
+            assert self.top_k_metric_name is not None
+            assert self.top_k_minimize_metric is not None
 
         # Create separate filenames for checkpoints & final trained model
-        self.final_ckpt_dir = os.path.join(self.experiment_dir, "models/final/")
+        self.ckpt_dir = os.path.join(self.experiment_dir, "models/")
         self.final_model_save_fname = os.path.join(
-            self.final_ckpt_dir, "final_" + seed_id
+            self.ckpt_dir, "final", "final_" + seed_id
         )
+        self.init_model_save_fname = os.path.join(
+            self.ckpt_dir, "init", "init_" + seed_id
+        )
+        self.init_model_saved = False
         if self.save_every_k_ckpt is not None:
             self.every_k_ckpt_list: List[str] = []
-            self.every_k_dir = os.path.join(self.experiment_dir, "models/every_k/")
+            self.every_k_dir = os.path.join(
+                self.experiment_dir, "models/every_k/"
+            )
             self.every_k_model_save_fname = os.path.join(
                 self.every_k_dir, "every_k_" + seed_id + "_k_"
             )
@@ -56,12 +70,19 @@ class ModelLog(object):
             )
 
         # Different extensions to model checkpoints based on model type
-        if self.model_type in ["torch", "tensorflow", "jax", "sklearn", "numpy"]:
+        if self.model_type in [
+            "torch",
+            "tensorflow",
+            "jax",
+            "sklearn",
+            "numpy",
+        ]:
             if self.model_type in ["torch", "tensorflow"]:
                 self.model_fname_ext = ".pt"
             elif self.model_type in ["jax", "sklearn", "numpy"]:
                 self.model_fname_ext = ".pkl"
             self.final_model_save_fname += self.model_fname_ext
+            self.init_model_save_fname += self.model_fname_ext
 
         # Initialize counter & lists for top k scores and storage time to track
         if reload:
@@ -76,16 +97,20 @@ class ModelLog(object):
 
     def setup_model_ckpt_dir(self):
         """Create separate sub-dirs for checkpoints & final trained model."""
-        os.makedirs(self.final_ckpt_dir, exist_ok=True)
+        os.makedirs(self.ckpt_dir, exist_ok=True)
         if self.save_every_k_ckpt is not None:
             os.makedirs(self.every_k_dir, exist_ok=True)
         if self.save_top_k_ckpt is not None:
             os.makedirs(self.top_k_dir, exist_ok=True)
 
-    def save(self, model, clock_to_track, stats_to_track):  # noqa: C901
+    def save(
+        self, model, clock_tracked: dict, stats_tracked: dict
+    ):  # noqa: C901
         """Save current state of the model as a checkpoint."""
         # If first model ckpt is saved - generate necessary directories
         self.model_save_counter += 1
+        if self.model_save_counter == 1:
+            os.makedirs(os.path.join(self.ckpt_dir, "final"), exist_ok=True)
         self.stored_every_k = False
         self.stored_top_k = False
         if self.model_save_counter == 1:
@@ -96,17 +121,23 @@ class ModelLog(object):
 
         # CASE 2: SEPARATE STORAGE OF EVERY K-TH LOGGED MODEL STATE
         if self.save_every_k_ckpt is not None:
-            self.save_every_k_model(model, clock_to_track)
+            self.save_every_k_model(model, clock_tracked)
 
         # CASE 3: STORE TOP-K MODEL STATES BY SOME SCORE
         if self.save_top_k_ckpt is not None:
-            self.save_top_k_model(model, clock_to_track, stats_to_track)
+            self.save_top_k_model(model, clock_tracked, stats_tracked)
+
+    def save_init_model(self, model):
+        """Store the initial model checkpoint and replace old ckpt."""
+        os.makedirs(os.path.join(self.ckpt_dir, "init"), exist_ok=True)
+        save_model_ckpt(model, self.init_model_save_fname, self.model_type)
+        self.init_model_saved = True
 
     def save_final_model(self, model):
         """Store the most recent model checkpoint and replace old ckpt."""
         save_model_ckpt(model, self.final_model_save_fname, self.model_type)
 
-    def save_every_k_model(self, model, clock_to_track):
+    def save_every_k_model(self, model, clock_tracked: dict):
         """Store every kth provided checkpoint."""
         if self.model_save_counter % self.save_every_k_ckpt == 0:
             ckpt_path = (
@@ -116,16 +147,16 @@ class ModelLog(object):
             )
             save_model_ckpt(model, ckpt_path, self.model_type)
             # Use latest update performance for last checkpoint
-            time = clock_to_track[self.ckpt_time_to_track].to_numpy()[-1]
+            time = clock_tracked[self.ckpt_time_to_track][-1]
             self.every_k_storage_time.append(time)
             self.every_k_ckpt_list.append(ckpt_path)
             self.stored_every_k = True
 
-    def save_top_k_model(self, model, clock_to_track, stats_to_track):
+    def save_top_k_model(self, model, clock_tracked: dict, stats_tracked: dict):
         """Store top-k checkpoints by performance."""
         # Use latest update performance for last checkpoint
-        score = stats_to_track[self.top_k_metric_name].to_numpy()[-1]
-        time = clock_to_track[self.ckpt_time_to_track].to_numpy()[-1]
+        score = stats_tracked[self.top_k_metric_name][-1]
+        time = clock_tracked[self.ckpt_time_to_track][-1]
 
         # Fill up empty top k slots
         if len(self.top_k_performance) < self.save_top_k_ckpt:
@@ -154,33 +185,44 @@ class ModelLog(object):
             self.top_k_performance[id_to_replace] = score
             self.top_k_storage_time[id_to_replace] = time
             ckpt_path = (
-                self.top_k_model_save_fname + str(id_to_replace) + self.model_fname_ext
+                self.top_k_model_save_fname
+                + str(id_to_replace)
+                + self.model_fname_ext
             )
             save_model_ckpt(model, ckpt_path, self.model_type)
             self.stored_top_k = True
 
     def reload(self):
         """Reload results from previous experiment run."""
-        reloaded_log = load_log(self.experiment_dir)
+        reloaded_log = load_log(self.experiment_dir,
+                                aggregate_seeds=False,
+                                reload_log=True)
         # Make sure to reload in results for correct seed
-        if reloaded_log.eval_ids is None:
-            meta_data = reloaded_log.meta
+        if reloaded_log.eval_ids[0] == "no_seed_provided":
+            meta_data = reloaded_log["no_seed_provided"].meta
         else:
             meta_data = reloaded_log[self.seed_id].meta
 
         # Reload counter & lists for top k scores and storage time to track
         if self.save_every_k_ckpt is not None:
-            self.every_k_ckpt_list = [ck.decode() for ck in meta_data.every_k_ckpt_list]
-            self.every_k_storage_time = meta_data.every_k_storage_time
+            if type(meta_data.every_k_ckpt_list) == list:
+                self.every_k_ckpt_list = [
+                    ck for ck in meta_data.every_k_ckpt_list
+                ]
+                self.every_k_storage_time = meta_data.every_k_storage_time
+            else:
+                self.every_k_ckpt_list = [meta_data.every_k_ckpt_list]
+                self.every_k_storage_time = [meta_data.every_k_storage_time]
+
             self.model_save_counter = int(
                 self.every_k_ckpt_list[-1].split(self.model_fname_ext)[0][-1]
             )
         else:
             self.model_save_counter = 0
         if self.save_top_k_ckpt is not None:
-            self.top_k_ckpt_list = [ck.decode() for ck in meta_data.top_k_ckpt_list]
+            self.top_k_ckpt_list = [ck for ck in meta_data.top_k_ckpt_list]
             self.top_k_storage_time = meta_data.top_k_storage_time
-            self.top_k_performance = meta_data.top_k_performance.tolist()
+            self.top_k_performance = meta_data.top_k_performance
 
 
 def save_model_ckpt(model, model_save_fname: str, model_type: str) -> None:
@@ -194,7 +236,9 @@ def save_model_ckpt(model, model_save_fname: str, model_type: str) -> None:
         # JAX/sklearn save parameter dict/model as dictionary
         save_pkl_object(model, model_save_fname)
     else:
-        raise ValueError("Provide valid model_type [torch, jax, sklearn, numpy].")
+        raise ValueError(
+            "Provide valid model_type [torch, jax, sklearn, numpy]."
+        )
 
 
 def save_torch_model(path_to_store: str, model) -> None:
